function experiment(config_file, datapath, datapath_t, d, a, b, options, seed)

if nargin == 6
	options = '';
	seed = 111;
elseif nargin ~= 8
	error('The #arguments is incorrect.');
end

addpath(genpath('simpleNN'));

%% Train
% ------
% Read train data sets
[y, Z] = libsvmread(datapath);
y = y - min(y) + 1;
Z = [full(Z) zeros(size(Z,1), a*b*d - size(Z,2))];

% Rearrange data from row-wise to col-wise
Z = reshape(permute(reshape(Z, [],b,a,d), [1,3,2,4]), [], a*b*d);

% Max-min normalization
tmp_max = max(Z, [], 2);
tmp_min = min(Z, [], 2);
Z = (Z - tmp_min) ./ (tmp_max - tmp_min);

% Zero mean
mean_tr = mean(Z);
Z = Z - mean_tr;

model = cnn_train(y, Z, config_file, options, seed);

%% Test
% -----
K = length(unique(y));

% Read test data sets
[y, Z] = libsvmread(datapath_t);
y = y - min(y) + 1;
Z = [full(Z) zeros(size(Z,1), a*b*d - size(Z,2))];

% Rearrange data from row-wise to col-wise
Z = reshape(permute(reshape(Z, [],b,a,d), [1,3,2,4]), [], a*b*d);

% Max-min normalization
tmp_max = max(Z, [], 2);
tmp_min = min(Z, [], 2);
Z = (Z - tmp_min) ./ (tmp_max - tmp_min);

% Zero mean
Z = Z - mean_tr;

[predicted_label, acc] = cnn_predict(y, Z, model);

% Calculate precision, recall and F1 measure
avg_precision = 0;
avg_recall = 0;
f1 = 0;
for k = 1 : K
	predict_k = predicted_label == k;
	y_k = y == k;
	tp = sum(predict_k & y_k);
	fp = sum(xor(predict_k, predict_k & y_k));
	fn = sum(~predict_k & y_k);

	% macro
	if tp ~= 0
		precision = tp / (tp + fp);
		recall = tp / (tp + fn);
		f1 = f1 + (2 * precision * recall) / (precision + recall);

		avg_precision = avg_precision + precision;
		avg_recall = avg_recall + recall;
	end
end

avg_precision = avg_precision / K;
avg_recall = avg_recall / K;
f1_measure = f1 / K;

% Used for logging
param = parse_options(struct, options);

if isfield(param, 'lr')
	fprintf('config: %s seed: %d lr: %g test_acc: %g test_precision: %g test_recall: %g test_f1_measure: %g\n', config_file, seed, param.lr, acc, avg_precision, avg_recall, f1_measure);
else
	fprintf('config: %s seed: %d test_acc: %g test_precision: %g test_recall: %g test_f1_measure: %g\n', config_file, seed, acc, avg_precision, avg_recall, f1_measure);
end


function param = parse_options(param, options)

options = strsplit(strtrim(options), ' ');

if mod(length(options), 2) == 1
	error('Each option is specified by its name and value.');
end

for i = 1 : length(options)/2
	option = options{2*i-1};
	value = str2num(options{2*i});
	switch option
		case '-s'
			param.solver = value;
		case '-SR'
			param.SR = value;
		case '-iter_max'
			param.iter_max = value;
		case '-C'
			param.C = value;
		case '-xi'
			param.xi = value;
		case '-CGmax'
			param.CGmax = value;
		case '-lambda'
			param.lambda = value;
		case '-drop'
			param.drop = value;
		case '-boost'
			param.boost = value;
		case '-eta'
			param.eta = value;
		case '-lr'
			param.lr = value;
		case '-decay'
			param.decay = value;
		case '-bsize'
			param.bsize = value;
		case '-momentum'
			param.momentum = value;
		case '-epoch_max'
			param.epoch_max = value;
		otherwise
			error('%s is not a supported option.', option);
	end
end

