#!/bin/bash
newton_dir="cpuversion/simpleNN/experiment/logs/newton"
sg_dir="gpuversion/simpleNN/experiment/logs/sgd"

g++ get_last_acc.cpp -o get_last_acc

data_arr=("mnist" "SVHN" "cifar10" "smallNORB-32x32")
C_arr=("0.01" "0.1" "1")
Clabel_arr=("0-01" "0-1" "1")
lambda_arr=("1")
SR_arr=("0-1" "0-05" "0-01")
bsize_arr=("128")
lr_arr=("0.1" "0.03" "0.01" "0.003" "0.001" "0.0003" "0.0001")
lrlabel_arr=("0-1" "0-03" "0-01" "0-003" "0-001" "0-0003" "0-0001")
seed_arr=("1" "2" "3" "4" "5")
epoch_max="1000"
iter_max="100"

for (( m = 0 ; m < ${#data_arr[@]}; m++ ))
do
	if [[ $m -eq 0 ]]
	then
		printf "%s\n" "\begin{table}[t]"
		printf "%s\n" "\caption{Each test accuracy is the average of five results by using the same initial solutions as in Table \ref{table:exps-results-CNN}. For SG, in some situations not all the five initial weights lead to the convergence.}"
		printf "%s\n" "\label{table:exps-robust}"
	fi

	printf "%s\n" "\begin{center}"
	printf "%s\n" "\begin{tabular}{>{\centering\arraybackslash}p{0.6cm} | >{\centering\arraybackslash}p{1.2cm} >{\centering\arraybackslash}p{1.2cm} >{\centering\arraybackslash}p{1.2cm} | >{\centering\arraybackslash}p{1.2cm} >{\centering\arraybackslash}p{1.2cm} >{\centering\arraybackslash}p{1.2cm} >{\centering\arraybackslash}p{1.2cm} >{\centering\arraybackslash}p{1.2cm} >{\centering\arraybackslash}p{1.2cm} >{\centering\arraybackslash}p{1.2cm}}"
	printf "%s\n" "\multicolumn{1}{c}{ } & \multicolumn{3}{c}{Sampling rate (Newton)} & \multicolumn{7}{c}{Initial learning rate (SG)}\\\\"
	printf "%s\n" "\hline"
	printf "%s\n" " \$C$\ & \$10\%$ & \$5\%$ & \$1\%$ & \$0.1$ & \$0.03$ & \$0.01$ & \$0.003$ & \$0.001$ & \$0.0003$ & \$0.0001$\\\\"

	for (( i = 0 ; i < ${#C_arr[@]}; i++))
	do
		printf "%s\n" "\hline"
		printf "$%s$ " "${C_arr[$i]}l"

		# Newton
		for (( k = 0 ; k < ${#SR_arr[@]}; k++))
		do
			for (( j = 0 ; j < ${#lambda_arr[@]}; j++))
			do
				acc=0
				success_count=0
				for (( t = 0 ; t < ${#seed_arr[@]}; t++))
				do
					current_file="${newton_dir}/${data_arr[$m]}-layer3-SR-${SR_arr[$k]}-C-${Clabel_arr[$i]}-lambda-${lambda_arr[$j]}-iter-${iter_max}-seed-${seed_arr[$t]}.log"
					if [[ -f ${current_file} ]]
					then
						acc_tmp=`./get_last_acc ${current_file}`
						if [[ ${acc_tmp} =~ ^[0-9]+([.][0-9]+)?$ ]]
						then
							acc=`echo "${acc} + ${acc_tmp}" | bc -l`
							success_count=`echo "${success_count} + 1" | bc`
						fi
					fi
				done
				if [[ ${success_count} -eq ${#seed_arr[@]} ]]
				then
					acc=`printf %.2f $(echo "scale=3;${acc}/${#seed_arr[@]}" | bc -l)`
				else
					acc=0
				fi
				printf "& \$%s\$ " "${acc}\\%"
			done
		done

		# SGD
		for (( k = 0 ; k < ${#bsize_arr[@]}; k++))
		do
			for (( j = 0 ; j < ${#lr_arr[@]}; j++))
			do
				acc=0
				success_count=0
				for (( t = 0 ; t < ${#seed_arr[@]}; t++))
				do
					current_file="${sg_dir}/${data_arr[$m]}-layer3-bsize-${bsize_arr[$k]}-C-${Clabel_arr[$i]}-lr-${lrlabel_arr[$j]}-epoch-${epoch_max}-mnt-0-9-decay-6-seed-${seed_arr[$t]}-whole.log"
					if [[ -f ${current_file} ]]
					then
						acc_tmp=`./get_last_acc ${current_file}`
						if [[ ${acc_tmp} =~ ^[0-9]+([.][0-9]+)?$ ]]
						then
							acc=`echo "${acc} + ${acc_tmp}" | bc -l`
							success_count=`echo "${success_count} + 1" | bc`
						fi
					fi
				done
				if [[ ${success_count} -eq ${#seed_arr[@]} ]]
				then
					acc=`printf %.2f $(echo "scale=3;${acc}/${#seed_arr[@]}" | bc -l)`
				else
					acc=0
				fi
				printf "& \$%s\$ " "${acc}\\%"
			done
		done
		printf "%s\n" "\\\\"
	done

	printf "%s\n" "\end{tabular}"
	if [[ ${data_arr[$m]} = "smallNORB-32x32" ]]
	then
		printf "%s\n" "\subcaption{{\sf smallNORB}.}"
	else
		printf "%s\n" "\subcaption{{\sf ${data_arr[$m]^^}}.}"
	fi
	printf "%s\n" "\end{center}"

	if [[ $m -eq `echo "${#data_arr[@]} - 1" | bc` ]]
	then
		printf "%s\n" "\end{table}"
	fi
done

